//***************************************************************************** 
//   -- RemoteXY Gygro 4.0 -- 
//   -- for ANDROID 3.7.1 or later version--
//       
// Pulse pin (D5): Connect to sensor via 100K resistor.
// Reference voltage of 0.632 of output pin (pulsePin) connected to D7.
// In my case I used 3.06V because I measured 4.84 on the 5V pin.
// Measure pin (D6) connected to first leg of capacitor, other leg connected to Gnd.
//
// Like this:
// Sensor
// D5  ----> 100K ----> D6 ----> sensor ----> Gnd
// Reference voltage:
// +5V ----> 36K ---> D7 ---> 62K ----> Gnd
// Bluetooth
// D2 ----> TxD Module
// D3 ----> RxD Module
//*******************************************************************************

#define Cpar 38.75;          // паразитная емкость ( при отключеном сенсоре)
#define Kf 0.2            // коэфициент фильтрации

// определение режима соединения и подключение библиотеки RemoteXY 
#define REMOTEXY_MODE__SOFTSERIAL    // режим передачи через программный UART
#include <SoftwareSerial.h>           // подключение библиотеки SoftwareSerial

#include <RemoteXY.h>                 // подключение библиотеки RemoteXY

// RemoteXY настройка соединения  
#define REMOTEXY_SERIAL_RX 2 
#define REMOTEXY_SERIAL_TX 3 
#define REMOTEXY_SERIAL_SPEED 9600 


// RemoteXY конфигурация интерфейса   
#pragma pack(push, 1) 
uint8_t RemoteXY_CONF[] = 
  { 0,33,119,0,6,5,2,67,4,32
  ,14,34,5,11,23,43,5,2,21,67
  ,4,33,38,18,5,28,60,20,5,2
  ,11,66,1,59,33,7,16,13,51,7
  ,24,2,129,0,30,5,29,6,14,15
  ,37,5,9,208,161,32,208,180,208,176
  ,209,130,209,135,208,184,208,186,208,176
  ,32,40,208,191,208,164,41,0,129,0
  ,23,24,53,6,4,39,55,5,9,208
  ,146,208,187,208,176,208,182,208,189,208
  ,190,209,129,209,130,209,140,32,208,191
  ,208,190,209,135,208,178,209,139,32,40
  ,37,41,0 }; 
   
// структура определяет все переменные интерфейса управления  
struct { 

    // переменные ввода вывода
  char text_1[21];  // строка UTF8 оканчивающаяся '0' 
  char text_2[11];  // строка UTF8 оканчивающаяся '0' 
  int8_t level_1;   // =0..100 положение уровня 

    // другие переменные
  uint8_t connect_flag;  // флаг подключения, =1 если подключено 

} RemoteXY; 
#pragma pack(pop)

// глобальные переменные
//*************************************
// требуют настройки на используемый сенсор
const float Cmin = 5.0;          // минимальная емкость (в воздухе)
const float Cmax = 127.0;         // максимальная емкость (погруженный в воду)
//**************************************


const byte pulsePin = 5;               // Пин импульса заряда D5
const float resistance = 100.0;        // Сопротивление зарядного резистора в килоомах

volatile boolean triggered;            // Конец измерения
volatile boolean active;               // Измерение активно

float Cres=0;                          // емкость результирующая 

// параметры кольцевого буфера
const int bufferSize = 20;             // Размер кольцевого буфера
unsigned long buffer[bufferSize];      // Кольцевой буфер
unsigned int bufferindex;
// переменные для таймера
unsigned long timerRes;                 // результирующее значение таймера после фильтрации 
volatile unsigned long timerCounts;    // Значение таймера счетчика
volatile unsigned long overflowCount;  // количество переполнений таймера счетчика


/***************/
// вектора прерываний
/****************/
ISR (TIMER1_OVF_vect)            // вектор прерывания по переполнению Timer 1
{
  ++overflowCount;               // увеличиваем счетчик переполнений  
}
/***************/

/***************/
ISR (ANALOG_COMP_vect)
  {
  // захватываем в этот момент значение таймера ( до его следующего изменения )
  unsigned int timer1CounterValue;
  timer1CounterValue = TCNT1;      // see datasheet, page 117 (accessing 16-bit registers)
  
  if (active)
    {
     if (TIFR1 & TOV1)             // если последнее переполнение пропущено
      overflowCount++;             // прибавляем счетчик переполнений
    timerCounts = (overflowCount << 16) + timer1CounterValue;  // вычисление значения счетчика
    triggered = true;              // завершаем активный режим
    digitalWrite (pulsePin, LOW);  // старт разрядки конденсатора
    }
  }

/***************/

/***************/
void setup()  
{ 
  RemoteXY_Init ();                // инициализация оболочки RemoteXY  
   
  pinMode (pulsePin, OUTPUT);      // pulsePin устанавливаем на выход
  digitalWrite (pulsePin, LOW);    // и обнуляем
  
  Serial.begin (9600);             //устанавливаем последовательное соединение
  ADCSRB = 0;                      // (Disable) ACME: Analog Comparator Multiplexer Enable
  ACSR =  _BV (ACI)                // очистка флага компаратора
        | _BV (ACIE)               // разрешение прерывания от компаратора
        | _BV (ACIS0) | _BV (ACIS1);  // ACIS1, ACIS0: прерывание по изменению с 0 на 1
   bufferindex = 0;                // обнуляем индекс кольцевого буфера     
} 
/***************/
// подпрограммы
/***************/
void OutRes()                      // вывод результатов измерений
{
  unsigned int humidity = 0;       // влажность  
 
  Cres = timerRes * 1000 / 16.0 / resistance - Cpar;   // вычисление С сенсора

// Вывод емкости
  Serial.print ("C sens = ");
  Serial.print (Cres);
  Serial.println (" pF   ");
    /*  
    преобразуем значение емкости в строку  
    и помещаем ее сразу в поле text_1 структуры RemoteXY  
  */ 
  itoa(Cres, RemoteXY.text_1, 10);
 // Вывод влажности
 if (Cres < Cmin)                  // отсутствие сенсора
   if (Cres > Cmin -2)             // пороговая зона 1пФ
    {
  Serial.println ("0%");
  strcpy (RemoteXY.text_2 , "0");
  RemoteXY.level_1 = 0;
    }
 
   else
    {
  Serial.println ("Error: NoSensor");
  strcpy (RemoteXY.text_2 , "NoSensor");
  RemoteXY.level_1 = 0;
    }
    else
  if ( Cres > Cmax )              // превышение значения емкости для настроенного сенсора
    if ( Cres < Cmax + 3 )        // пороговая зона 2пФ
    {
  Serial.println ("100%");
  strcpy (RemoteXY.text_2 , "Excess");
  RemoteXY.level_1 = 100;
    }
    else
    { 
  Serial.println ("Error: Excess");
  strcpy (RemoteXY.text_2 , "Excess");
  RemoteXY.level_1 = 100;
    }
  else                             // значение в норме
  {
  humidity = (Cres - Cmin) * 100 / (Cmax - Cmin);
  Serial.print ("Humidity = ");

   Serial.print (humidity);
  Serial.println ("%");
 
  itoa(humidity, RemoteXY.text_2, 10);
  RemoteXY.level_1 = humidity;
  }
}

/***************/

/***************/
void startTiming ()                // программа старта измерения
  {
  active = true;                   // выставляем флаг активности
  triggered = false;               // флаг конца измерения сбрасываем

  // настройка таймера
  overflowCount = 0;               // обнуляем счетчик переполнений
  // сброс Timer 1
  TCCR1A = 0;             
  TCCR1B = 0;   
  TCNT1 = 0;                       // Обнуление Timer 1
  // Timer 1 - подсчет тактовых импульсов
  TIMSK1 = _BV (TOIE1);            // разрешаем прерывание по переполнению Timer 1

   digitalWrite (pulsePin, HIGH);  // старт зарядки сенсора
 
  TCCR1B =  _BV (CS10);	           // старт Timer 1 без предделителя
    
  } 
/***************/

/***************/
void finishTiming ()               // программа завершения измерения          
  {
  active = false;                  // обнуляем флаг активности
  triggered = false;                // обнуляем флаг конца измерения
  } 
/***************/

/***************/
void loop()  
{  
  RemoteXY_Handler (); 
   
  if (!active)                        // если измерение не активно
    startTiming ();                   // запуск измерения
    
  if (active && triggered)            // если достигли момента конца измерения
   { 
     finishTiming ();                 // завершаем измерение
      buffer[bufferindex] = timerCounts;      // заполняем кольцевой буфер
      bufferindex++;                    
      if (bufferindex >= bufferSize)  // буфер заполнен
      {
       bufferindex = 0;               // обнуляем индекс кольцевого буфера
         float sum = 0;
       for (int i = 0; i < bufferSize; i++) // Кольцевой фильтр
         {
          sum += buffer[i];
          }
       timerRes = (1-Kf) * timerRes  + Kf * sum / bufferSize;         // НЧ фильтр
       OutRes();                        // вывод результатов измерений
      delay ( 500 );                    // задержка 0.5 сек 
      } 
   }
    
}
/***************/
